@@ -2,21 +2,27 @@ package Net::Amazon::DynamoDB::Marshaler;
 
 use strict;
 use 5.008_005;
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use parent qw(Exporter);
 our @EXPORT = qw(dynamodb_marshal dynamodb_unmarshal);
 
 use boolean qw(true false isBoolean);
-use Scalar::Util qw(looks_like_number blessed);
+use Scalar::Util qw(blessed);
+use Types::Standard qw(StrictNum);
 
 sub dynamodb_marshal {
-    my ($attrs) = @_;
+    my ($attrs, %args) = @_;
+    my $force_type = $args{force_type} || {};
     die __PACKAGE__.'dynamodb_marshal(): argument must be a hashref' unless (
         ref $attrs
         && ref $attrs eq 'HASH'
     );
-    return _marshal_hashref($attrs);
+    die __PACKAGE__.'dynamodb_marshal(): force_type must be a hashref' unless (
+        ref $force_type
+        && ref $force_type eq 'HASH'
+    );
+    return _marshal_hashref($attrs, $force_type);
 }
 
 sub dynamodb_unmarshal {
@@ -29,8 +35,12 @@ sub dynamodb_unmarshal {
 }
 
 sub _marshal_hashref {
-    my ($attrs) = @_;
-    return { map { $_ => _marshal_val($attrs->{$_}) } keys %$attrs };
+    my ($attrs, $force_type) = @_;
+    $force_type ||= {};
+    return {
+        map { $_ => _marshal_val($attrs->{$_}, $force_type->{$_}) }
+        keys %$attrs
+    };
 }
 
 sub _unmarshal_hashref {
@@ -39,8 +49,8 @@ sub _unmarshal_hashref {
 }
 
 sub _marshal_val {
-    my ($val) = @_;
-    my $type = _val_type($val);
+    my ($val, $type) = @_;
+    $type ||= _val_type($val);
 
     return { $type => $val } if $type =~ /^(N|S)$/;
     return { $type => 1 } if $type eq 'NULL';
@@ -99,7 +109,7 @@ sub _is_number {
     my ($val) = @_;
     return (
         (!ref $val)
-        && looks_like_number($val)
+        && StrictNum->check($val)
         && (
             $val == 0
             || (
@@ -176,7 +186,7 @@ If the value is undef or an empty string, use Null ('NULL').
 
 =item 2.
 
-If the value looks like a number, and falls within the accepted range for a DynamoDB number, use Number ('N').
+If the value is a number (per StrictNum in L<Types::Standard>), and falls within the accepted range for a DynamoDB number, use Number ('N').
 
 =item 3.
 
@@ -196,7 +206,7 @@ If the value isa L<boolean>, use Boolean ('BOOL').
 
 =item 7.
 
-If the value isa L<Set::Object>, use either Number Set ('NS') or String Set ('SS'), depending on whether all members look like numbers or not. All members must be defined, non-reference values, or an error will be thrown.
+If the value isa L<Set::Object>, use either Number Set ('NS') or String Set ('SS'), depending on whether all members are numbers or not. All members must be defined, non-reference values, or an error will be thrown.
 
 =item 8.
 
@@ -214,7 +224,34 @@ By default, dynamodb_marshal and dynamodb_unmarshal are exported.
 
 Takes in a "normal" Perl hashref, transforms it into DynamoDB format.
 
-  my $attrs_marshalled = dynamodb_marshal($attrs);
+  my $attrs_marshalled = dynamodb_marshal($attrs[, force_type => {}]);
+
+=head3 force_type
+
+Sometimes you want to explicitly choose a type for an attribute, overridding the rules above. Most commonly this issue occurs for key attributes, as DynamoDB enforces consistent typing on these attributes that it doesn't enforce otherwise.
+
+For instance, you might have a table named 'users' whose partition key is a string named 'username'. If you have incoming data with a username of '1234', this module will tell DynamoDB to store that as a number, which will result in an error.
+
+Use force_type in that situation:
+
+  my $item = {
+      username => '1234',
+      ...
+  };
+
+  my $force_type = {
+      username => 'S',
+  };
+
+  my $item_dynamodb = dynamodb_marshal($item, force_type => $force_type);
+
+  # $item_dynamodb looks like:
+  # {
+  #   username => {
+  #     S => '1234',
+  #   },
+  #   ...
+  # };
 
 =head2 dynamodb_unmarshal
 
